<?php
/**
 * Adamspay
 *
 * @author    Adamspay <www.adamspay.com>
 */

class AdamspayDeuda extends ObjectModel
{
    public $row_id;
    public $url_pago;
    public $id_cart;
    public $id_orden;
    public $id_deuda;
    public $estado_orden;
    public $estado_deuda;
    public $creado;
    public $actualizado;

    CONST DEUDA_PAGADA = 'paid';
    CONST DEUDA_PENDIENTE = 'pending';
    const DEUDA_CANCELADA = 'canceled';
    const ORDEN_PAGADA = 11;

    public static $definition = array(
        'table' => 'adamspay_deuda',
        'primary' => 'row_id',
        'fields' => array(
            'url_pago' => array('type' => self::TYPE_STRING, 'validate' => 'isString', 'required' => true),
            'id_cart' => array('type' => self::TYPE_INT, 'validate' => 'isUnsignedId', 'required' => true),
            'id_orden' => array('type' => self::TYPE_INT, 'validate' => 'isUnsignedId', 'required' => false),
            'id_deuda' => array('type' => self::TYPE_STRING, 'validate' => 'isString', 'required' => true),
            'estado_orden' => array('type' => self::TYPE_STRING, 'required' => false),
            'estado_deuda' => array('type' => self::TYPE_STRING, 'validate' => 'isString', 'required' => true),
            'creado' => array('type' => self::TYPE_DATE, 'validate' => 'isDate'),
            'actualizado' => array('type' => self::TYPE_DATE, 'validate' => 'isDate'),
        ),
    );

    /**
     * @param int $id_cart
     * 
     * @return array
     */
    public static function obtenerDeudaPorIdCart($id_cart)
    {
        $sql = new DbQuery();
        $sql->select('row_id, id_deuda, estado_deuda, estado_orden');
        $sql->from(self::$definition['table']);
        $sql->where('`id_cart` = "'.pSQL($id_cart).'"');

        return Db::getInstance(_PS_USE_SQL_SLAVE_)->getRow($sql);
    }

    /**
     * @param int $id_cart
     * 
     * @return string row_id
     */
    public static function obtenerRowIdPorIdCart($id_cart)
    {
        $sql = new DbQuery();
        $sql->select('row_id');
        $sql->from(self::$definition['table']);
        $sql->where('`id_cart` = "' . pSQL($id_cart) . '"');

        return Db::getInstance(_PS_USE_SQL_SLAVE_)->getValue($sql);
    }

    /**
     * @param int $id_orden
     * 
     * @return string row_id
     */
    public static function obtenerRowIdPorIdOrden($id_orden)
    {
        $sql = new DbQuery();
        $sql->select('row_id');
        $sql->from(self::$definition['table']);
        $sql->where('`id_orden` = "' . pSQL($id_orden) . '"');

        return Db::getInstance(_PS_USE_SQL_SLAVE_)->getValue($sql);
    }

    /**
     * @param int $id_orden
     * 
     * @return array
     */
    public static function obtenerDeudaPorIdOrden($id_orden)
    {
        $sql = new DbQuery();
        $sql->select('row_id, id_deuda, estado_deuda, estado_orden, url_pago');
        $sql->from(self::$definition['table']);
        $sql->where('`id_orden` = "' . pSQL($id_orden) . '"');

        return Db::getInstance(_PS_USE_SQL_SLAVE_)->getRow($sql);
    }

    /**
     * @param int $id_deuda
     * 
     * @return array
     */
    public static function obtenerDeudaPorIdDeuda($id_deuda)
    {
        $sql = new DbQuery();
        $sql->select('row_id, id_deuda, estado_deuda, estado_orden, url_pago, id_cart, id_orden');
        $sql->from(self::$definition['table']);
        $sql->where('`id_deuda` = "' . pSQL($id_deuda) . '"');

        return Db::getInstance(_PS_USE_SQL_SLAVE_)->getRow($sql);
    }
}
