<?php
namespace Adamspay;

require_once(_PS_ROOT_DIR_ . '/config/config.inc.php');
require_once(_PS_ROOT_DIR_ . '/init.php');
require_once(_PS_ROOT_DIR_ . '/modules/adamspay/adamspay.php');
require_once(_PS_ROOT_DIR_ . '/modules/adamspay/classes/AdamspayApi.php');

use Configuration;
use Adamspay\AdamspayApi;

class AdamspayHelper {
    /**
     * Se usa en el método install() y en update vía post
     * Acorta el parametro PS_SHOP_NAME para poder setear en la configuración del prefijo.
     * Pone en minuscula, recorta hasta 10 caracteres.
     * Ejemplo: "AdamsPay - Online Store" >> "adamspay_o"
     * Luego puede cambair.
     * */

    // @todo [Migs] @done
    // a) El prefijo debe ser configurable y el valor de acotarNombreTienda() utilizado como DEFAULT si no se configuró.
    // b) El prefijo se utilizaría sólo al crear la deuda la 1era vez y luego el ID de la deuda almacenada y asociada
    //    a la orden para evitar fallas en caso que se cambie el prefijo (o nombre del sitio).

    public static function acortarNombreTienda($tieneParametro = null)
    {
        // Dejamos solo letras y numeros.
        $buscar = ['\'', '"', ' ', '-', '_', '.', '*', '!', '@', '$', '%', '^', '(', ')', '+', '=', '/', '?', '<', '>', '|'];

        if ($tieneParametro) {
            // Si tiene parametro, no usamos nombre de tienda, usamos el parametro para limpiar
            $stringTrabajar = str_replace($buscar, '', $tieneParametro);
        } else {
            $stringTrabajar = str_replace($buscar, '', Configuration::get('PS_SHOP_NAME'));
        }

        if (strlen($stringTrabajar) > 10) {
            $stringTrabajar = substr($stringTrabajar, 0, 10);
        }

        // strripos buscamos la ultima aparición de '_' Guion. Previamente ya limpiamos que no contenga
        // en el String.
        if (!strripos($stringTrabajar, '_')) {
            $stringTrabajar = $stringTrabajar . '_';
        }

        return strtolower($stringTrabajar);
    }

    /**
     * Formatea los decimales, segun codigo ISO de moneda.
     * @todo: se puede obtener directo los decimales configurados.
     * */
    public static function formatearMoneda($montoFormatear, $moneda)
    {
        $montoFormateado = null;
        if ($moneda == 'PYG') {

            $montoFormateado = number_format($montoFormatear, 0, '', '');
        } elseif ($moneda == 'USD') {
            $montoFormateado = number_format($montoFormatear, 2);
        } elseif ($moneda == 'EUR') {
            $montoFormateado = number_format($montoFormatear, 2);
        }

        return $montoFormateado;
    }

    /**
     * busca el id orden dentro del id deuda retornado por adamspay
     * @param mixed $id_deuda
     * @return int
     */
    public static function verificarIdOrden($id_deuda){
        $posicion = strrpos($id_deuda, "_");
        if(!$posicion){
            return $posicion;   // [TODO] Migs Por qué retorna $posicion? (no debería ser false?)
        }
        $posicion++;//para omitir el "_"

        $orderId = substr($id_deuda, $posicion);
        return is_numeric($orderId) ? $orderId : false;
    }

    /**
     * @param mixed $post post request data
     * @param string $http_hash
     * 
     * @return boolean
     */
    public static function validateHeaderNotification($post, $http_hash)
    {
        // local_hash = provider_string + post_data + client_secret
        $local_hash = md5('adams' . $post . Configuration::get('ADAMSPAY_API_SECRET_APP'));

        // [Migs] cambiado a triple === para evitar conversiones ($http_hash=true o $http_hash=0, ambos pasarían el test)
        return $local_hash === $http_hash;  
    }

    /**
     * Valida que la configuración cargada coincida con la configuración existente en adamspay
     * @param array $configuraciones
     * 
     * @return mixed
     */
    public static function validarConfiguracionApp($configuraciones){
        $errores = [];

        if (empty($configuraciones['ADAMSPAY_API_KEY_APP'])){
            $errores[] = 'El parámetro Api Key está vacío';
            return $errores;
        }

        if (empty($configuraciones['ADAMSPAY_API_SECRET_APP'])) {
            $errores[] = 'El parámetro Api Secret está vacío';
            return $errores;
        }

        $adamspay = new AdamspayApi;
        
        $informacion_app = $adamspay->obtenerInformacionApp();
        // [TODO] @done Migs: $informacion_app podría ser false: No se maneja ese caso
        if(!$informacion_app){
            $errores[] = 'Ha ocurrido un error al intentar obtener información de su "Aplicacion" de Adamspay.com';
            return $errores;
        }
        
        $api_secret_error = 'El ApiSecret ingresado no coincide con el ApiSecret proveído por AdamsPay.com';
        if($informacion_app['meta']['status'] == 'error'){
            $errores[] = ($informacion_app['meta']['code'] == 'bad.secret.hash') ? $api_secret_error : $informacion_app['meta']['description'];
            return $errores;
        }

        if(empty($informacion_app['app']['returnUrls'])){
            $errores[] = 'Debe de configurar la url de respuesta dentro de Adamspay.com';
        }

        if(isset($informacion_app['app']['returnUrls']['default']) && $informacion_app['app']['returnUrls']['default'] <> $configuraciones['ADAMSPAY_URL_RESPUESTA']){
            $errores[] = 'La url de respuesta configurada en Adamspay.com no coincide con la configuración del módulo';
        }

        if (empty($informacion_app['app']['notifyUrls'])) {
            $errores[] = 'Debe de configurar la url del webhook dentro de Adamspay.com';
        }

        if (isset($informacion_app['app']['notifyUrls']['default']) && $informacion_app['app']['notifyUrls']['default'] <> $configuraciones['ADAMSPAY_URL_WEBHOOK']) {
            $errores[] = 'La url del webhook configurada en Adamspay.com no coincide con la configuración del módulo';
        }

        if(!empty($errores)){
            return $errores;
        }

        return true;
    }

    /**
     * @param mixed $data
     * 
     * @return string
     */
    public static function base64url_encode($data)
    {
        return rtrim(strtr(base64_encode($data), '+/', '-_'), '=');
    }
}