<?php
require_once(_PS_ROOT_DIR_ . '/modules/adamspay/classes/AdamspayDeuda.php');
require_once(_PS_ROOT_DIR_ . '/modules/adamspay/classes/AdamspayLog.php');

use Adamspay\AdamspayApi;
use Adamspay\AdamspayLog;

class AdamspayRedirectModuleFrontController extends ModuleFrontController
{
    
    public function postProcess()
    {
        $this->context = Context::getContext();
        
        if (Tools::getValue('action') == 'error') {
            
            $message = !empty(Tools::getValue('mensaje')) ? Tools::getValue('mensaje') : 'Ha ocurrido un error al hacer el redireccionamiento';
            $id_orden = !empty(Tools::getValue('id_orden')) ? Tools::getValue('id_orden') : null;
            $customer = !empty(Tools::getValue('id_customer')) ? new Customer(Tools::getValue('id_customer')) : null;
            return $this->displayError($message, $id_orden, $customer);

        } else {

            if (Configuration::get('ADAMSPAY_API_KEY_APP') && Configuration::get('ADAMSPAY_API_SECRET_APP')){
                $total = (string)$this->context->cart->getOrderTotal();
                $cart = $this->context->cart;
                $currency = $this->context->currency;
                $this->validateOrder($cart);

                if (!$this->isAuthorized()) {
                    die($this->trans('This payment method is not available.', array(), 'Modules.Checkpayment.Shop'));
                }

                $customer = new Customer($cart->id_customer);
                if (!Validate::isLoadedObject($customer)) {
                    Tools::redirect('index.php?controller=order&step=1');
                }

                $currency = $this->context->currency;
                $total = (float)$cart->getOrderTotal(true, Cart::BOTH);

                // Validamos la orden y obtenemos el $order_id
                $this->module->validateOrder(
                    (int) $cart->id,
                    Configuration::get('ADAMSPAY_PAGO_PENDIENTE'),
                    $total,
                    $this->module->displayName,
                    null,
                    null,
                    (int)$currency->id,
                    false,
                    $customer->secure_key
                );

                $id_orden = Order::getIdByCartId((int) $cart->id);

                $api = new AdamspayApi;
                $response = $api->crearDeuda($total, $id_orden, $currency->iso_code);

                if ($response) {
                    $url_pago = isset($response['debt']) ? $response['debt']['payUrl'] : null;
                    if ($url_pago) {
                        $adamspay_deuda = new AdamspayDeuda();
                        $adamspay_deuda->url_pago = (string)$url_pago;
                        $adamspay_deuda->id_cart = (int)$this->context->cart->id;
                        $adamspay_deuda->id_orden = (int)$id_orden;
                        $adamspay_deuda->id_deuda = (string)$response['debt']['docId'];
                        $adamspay_deuda->estado_orden = Configuration::get('ADAMSPAY_PAGO_PENDIENTE');
                        $adamspay_deuda->estado_deuda = (string)$response['debt']['payStatus']['status'];
                        $adamspay_deuda->creado = date('Y-m-d H:i:s');
                        $adamspay_deuda->actualizado = date('Y-m-d H:i:s');

                        if ($adamspay_deuda->save()) {
                            Tools::redirect($url_pago);
                        } else {
                            // Llegado a este punto, hacemos la redirección con el mensaje de error
                            $mensaje = 'Su deuda no pudo ser almacenada correctamente antes de redireccionarlo a la web de AdamsPay.';
                            AdamspayLog::error($mensaje, __FUNCTION__);
                            return $this->displayError($mensaje, $id_orden, $customer);
                        }
                    }
                }

                // Llegado a este punto, hacemos la redirección con el mensaje de error
                $mensaje = 'Su pago no ha sido confirmado, intente de nuevo con el Link de Pago';
                AdamspayLog::error($mensaje, __FUNCTION__);
                return $this->displayError($mensaje, $id_orden, $customer);
                
            }else{
                $mensaje = 'Debe configurar correctamente las credenciales de Adamspay antes de comenzar a recibir pagos.';
                AdamspayLog::error($mensaje, __FUNCTION__);
                return $this->displayError($mensaje);
            }
        }
    }

    /**
     * @param string $message
     * @param null $id_orden
     * @param null $customer
     * 
     * @return [type]
     */
    protected function displayError($message, $id_orden = null, $customer = null)
    {
        $this->errors[] = $this->module->l($message);
        $orden = $this->checkExistsOrder($id_orden);
        if($orden){
            if ($customer->is_guest) {
                $url_redirect = 'index.php?controller=guest-tracking&order_reference=' . $orden->reference . '&email=' . $customer->email;
            } else {
                $url_redirect = 'index.php?controller=order-confirmation&id_cart=' . (int)$orden->id_cart . '&id_module=' . (int)$this->module->id . '&id_order=' . $id_orden . '&key=' . $customer->secure_key;
            }
        }else{
            $url_redirect = 'index.php?controller=order&step=3';
        }

        $this->redirectWithNotifications($url_redirect);
    }

    /**
     * @param mixed $cart
     * 
     * @return boolean
     */
    protected function validateOrder($cart)
    {
        if ($cart->id_customer == 0 || $cart->id_address_delivery == 0 || $cart->id_address_invoice == 0 || !$this->module->active) {
            Tools::redirect('index.php?controller=order&step=1');
        }

        return true;
    }

    /**
     * @return boolean
     */
    protected function isAuthorized()
    {
        // Check that this payment option is still available in case the customer changed his address just before the end of the checkout process
        $authorized = false;
        foreach (Module::getPaymentModules() as $module) {
            if ($module['name'] == 'adamspay') {
                $authorized = true;
                break;
            }
        }

        return $authorized;
    }

    /**
     * @param mixed $id_orden
     * 
     * @return mixed
     */
    protected function checkExistsOrder($id_orden){
        return !empty($id_orden) ? new Order($id_orden) : false;
    }
}
