<?php
/**
* 2007-2020 PrestaShop
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Do not edit or add to this file if you wish to upgrade PrestaShop to newer
* versions in the future. If you wish to customize PrestaShop for your
* needs please refer to http://www.prestashop.com for more information.
*
*  @author    PrestaShop SA <contact@prestashop.com>
*  @copyright 2007-2020 PrestaShop SA
*  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*  International Registered Trademark & Property of PrestaShop SA
*/
require_once(_PS_ROOT_DIR_ . '/modules/adamspay/classes/AdamspayApi.php');
require_once(_PS_ROOT_DIR_ . '/modules/adamspay/classes/AdamspayLog.php');
require_once(_PS_ROOT_DIR_ . '/modules/adamspay/classes/AdamspayDeuda.php');
require_once(_PS_ROOT_DIR_ . '/modules/adamspay/classes/AdamspayHelper.php');

use Adamspay\AdamspayApi;
use Adamspay\AdamspayLog;
use Adamspay\AdamspayHelper;

class AdamspayValidationModuleFrontController extends ModuleFrontController
{
    
    public function postProcess()
    {
        $intent = !empty(Tools::getValue('intent')) && is_string(Tools::getValue('intent')) ? Tools::getValue('intent') : null;
        $id_deuda_adamspay = !empty(Tools::getValue('doc_id')) && is_string(Tools::getValue('doc_id')) ? Tools::getValue('doc_id') : null;
        $id_orden = AdamspayHelper::verificarIdOrden($id_deuda_adamspay);
        $deuda_local = AdamspayDeuda::obtenerDeudaPorIdDeuda($id_deuda_adamspay);

        if($intent === 'pay-debt' && $id_orden && !empty($deuda_local)){
            $orden = new Order($id_orden);
            $id_cart = $deuda_local['id_cart'];

            $customer = new Customer($orden->id_customer);
            
            //[Migs] Cambié el ORDEN de la expresión del IF para evitar hacer una llamada al API si la deuda local ya está pagada
            if($deuda_local['estado_deuda'] <> AdamspayDeuda::DEUDA_PAGADA && $this->verificarDeudaRemotaPagada($deuda_local['id_deuda']) ){
                $deuda_local = new AdamspayDeuda($deuda_local['row_id']);
                $deuda_local->estado_deuda = AdamspayDeuda::DEUDA_PAGADA;
                $deuda_local->estado_orden = AdamspayDeuda::ORDEN_PAGADA;
                $deuda_local->actualizado = date('Y-m-d H:i:s');
                $deuda_local->save();
            }

            if ($customer->is_guest) {
                Tools::redirect('index.php?controller=guest-tracking&order_reference=' . $orden->reference . '&email=' . $customer->email);
            } else {
                Tools::redirect('index.php?controller=order-confirmation&id_cart=' . (int)$id_cart . '&id_module=' . (int)$this->module->id . '&id_order=' . $id_orden . '&key=' . $customer->secure_key);
            }
        }else{
            $mensaje = 'La respuesta obtenida por parte de AdamsPay no es válida ';
            AdamspayLog::error($mensaje, __FUNCTION__);
            Tools::redirect($this->context->link->getModuleLink($this->module->name, 'redirect', ['action' => 'error', 'mensaje' => $mensaje], true));
        }
    }

    /**
     * @param string $id_deuda
     * 
     * @return boolean
     */
    protected function verificarDeudaRemotaPagada($id_deuda){
        $adamspay = new AdamspayApi;
        if($deuda_remota = $adamspay->obtenerDetalleDeuda($id_deuda)){
            return $deuda_remota['debt']['payStatus']['status'] === AdamspayDeuda::DEUDA_PAGADA;
        }

        return false;
    }
}
