<?php

ini_set('display_errors', 'on');
error_reporting(E_ALL);

ob_start();
require_once(_PS_ROOT_DIR_ . '/config/config.inc.php');
require_once(_PS_ROOT_DIR_ . '/init.php');
require_once(_PS_ROOT_DIR_ . '/modules/adamspay/adamspay.php');
require_once(_PS_ROOT_DIR_ . '/modules/adamspay/classes/AdamspayDeuda.php');

use Adamspay\AdamspayHelper;
use Adamspay\AdamspayLog;

class AdamspayWebhookModuleFrontController extends ModuleFrontController {
    public function initContent()
    {
        $this->ajax = true;

        /*
        * Validación AdamsPay.com
        * Más info: https://wiki.adamspay.com/devzone:concepts:webhook#validacion
        * */
        $post_request = file_get_contents('php://input');
        $datos_json = json_decode($post_request);

        if (AdamspayHelper::validateHeaderNotification($post_request, @$_SERVER['HTTP_X_ADAMS_NOTIFY_HASH'])) {
            // Obtener solo el Status
            if (isset($datos_json->notify) &&
                isset($datos_json->notify->type) &&
                $datos_json->notify->type === 'debtStatus')
            {
                $id_deuda = $datos_json->debt->docId;
                $id_orden = AdamspayHelper::verificarIdOrden($id_deuda);
                $estado_deuda = $datos_json->debt->payStatus->status;

                if ($estado_deuda === AdamspayDeuda::DEUDA_PAGADA && $id_orden) {
                    $order = new Order($id_orden);
                    if($order->current_state <> Configuration::get('PS_OS_CANCELED')){
                        $history = new OrderHistory();
                        $history->id_order = (int)$order->id;

                        // Ponemos el estado de la orden a Pago remoto aceptado.
                        $history->changeIdOrderState(Configuration::get('PS_OS_WS_PAYMENT'), (int)($order->id));
                        $history->add(true);

                        $row_id = AdamspayDeuda::obtenerRowIdPorIdOrden($id_orden);
                        $deuda_local = new AdamspayDeuda($row_id);
                        $deuda_local->estado_deuda = AdamspayDeuda::DEUDA_PAGADA;
                        $deuda_local->estado_orden = AdamspayDeuda::ORDEN_PAGADA;
                        $deuda_local->actualizado = date('Y-m-d H:i:s');

                        if($deuda_local->save()){
                            AdamspayLog::info('La deuda '.$id_deuda.' del id_orden '.$id_orden.' ha cambiado al estado '. AdamspayDeuda::DEUDA_PAGADA, __FUNCTION__);
                            http_response_code(200);
                            echo Tools::jsonEncode(array('code' => 200));
                            return;
                        }
                    }
                }

                AdamspayLog::error('estado_deuda:'.$estado_deuda.' - id_orden:'.$id_orden, __FUNCTION__);
                http_response_code(400);
                echo Tools::jsonEncode(array('code' => 400));
                return;
            } else {
                AdamspayLog::error('Variable $datos_json invalida.', __FUNCTION__);
                http_response_code(404);
                echo Tools::jsonEncode(array('code' => 404));
                return;
            }
        } else {
            AdamspayLog::error('Hash invalido.', __FUNCTION__);
            http_response_code(502);
            echo Tools::jsonEncode(array('code' => 502));
            return;
        }
    }
}
